/*
 * [[ MicroFB : Frozen Bubble for mobile phones ]]
 *
 * Copyright (c) 2000-2005 Guillaume Cottenceau.
 * Java sourcecode - Copyright (c) 2005 Glenn Sanson.
 *                                 2006 Adrian Kurek 
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Alexis Younes <73lab at free.fr>
 *      (everything but the bubbles)
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Guillaume Cottenceau <guillaume.cottenceau at free.fr>
 *      (design and manage the project, whole Perl sourcecode)
 *
 * Java version:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *
 *          [[ http://glenn.sanson.free.fr/microfb/ ]]
 *          [[ http://www.frozen-bubble.org/   ]]
 *
 * Support all devices:
 *    Adrian Kurek <wpqrek@wp.pl>
 *
 */

import javax.microedition.midlet.MIDlet;
import javax.microedition.lcdui.*;

public abstract class Base extends Canvas implements Runnable, CommandListener {

	private boolean init;
	private boolean run;
	
	private Image bgImg;
	private Graphics bgGfx;
	
	private MIDlet midlet;
	
	public Base(MIDlet midlet) {
		this.midlet = midlet;
		
		initClass();
		setCommandListener(this);
		
		init = true;
		run = true;
		new Thread(this).start();
	}
	
	protected final MIDlet getMIDlet() {
		return midlet;
	}
	
	protected final void paint(Graphics g) {
		if (isDoubleBuffered()) {
                        if (init)
                        {
                            g.setColor(0x000000);
                            g.fillRect(0,0, getWidth(), getHeight());
                        }
                        g.translate((getWidth()-128)>>1, (getHeight()-96)>>1);
			if (init) {
				initDraw(g);
			}
			draw(g);
                        g.translate(-g.getTranslateX(), -g.getTranslateY());
		}
		else {
			if (init) {
				if (bgImg == null) {
					bgImg = Image.createImage(128, 96);
					bgGfx = bgImg.getGraphics();
				}
                                g.setColor(0x000000);
                                g.fillRect(0,0, getWidth(), getHeight());
				initDraw(bgGfx);
			}
			draw(bgGfx);
			g.drawImage(bgImg,(getWidth()-128)>>1, (getHeight()-96)>>1, Graphics.TOP | Graphics.LEFT);
		}
		
		init = false;
	}
	
	protected final void stopDisplay() {
		run = false;
	}

	public final void run() {
		long time;
		
		while(run) {
			// init
			time = System.currentTimeMillis();
						
			// Action 
			action();

			if (run) {
				// Refresh
				repaint();
				
				// Reset
				time = System.currentTimeMillis() - time;
				
				if (time < 65) {
					try {
						Thread.sleep(65 - time);
					}
					catch(Exception e) {
					}
				}
			}
		}
	}
	
	public final void setInit(boolean value) {
		init = value;
	}

	public final boolean getInit() {
		return init;
	}	
	
	public void commandAction(Command c, Displayable d) {
		manageCommand(c);
	}
		
	public abstract void initClass();	
	
	public abstract void initDraw(Graphics g);
	
	public abstract void draw(Graphics g);
		
	public abstract void action();
	
	public abstract void manageCommand(Command c);
}
