/*
 * [[ MicroFB : Frozen Bubble for mobile phones ]]
 *
 * Copyright (c) 2000-2005 Guillaume Cottenceau.
 * Java sourcecode - Copyright (c) 2005 Glenn Sanson.
 *                                 2006 Adrian Kurek 
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Alexis Younes <73lab at free.fr>
 *      (everything but the bubbles)
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Guillaume Cottenceau <guillaume.cottenceau at free.fr>
 *      (design and manage the project, whole Perl sourcecode)
 *
 * Java version:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *
 *          [[ http://glenn.sanson.free.fr/microfb/ ]]
 *          [[ http://www.frozen-bubble.org/   ]]
 *
 * Support all devices:
 *    Adrian Kurek <wpqrek@wp.pl>
 *
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStore;

import java.util.*;

public class Game extends Base {
	private final static byte[][] BALL_DEF = {{2, 0, 4, 1}, {1, 1, 6, 1}, {0, 2, 8, 4}, {1, 6, 6, 1}, {2, 7, 4, 1}};
	
	private byte[][] launcher;
	private int[][] ballMove;
	
	private final static String COMMAND_TRY = "Try again";
	private final static String COMMAND_OVER = "Game over";
	
	private final static byte CHECK_NEXT_TURN = 1;
	private final static byte PART_OF_SET = 2;
	private final static byte LINKED_TO_TOP = 3;
	private final static byte REMOVE_BALL = 4;
		
	private final static byte END_LEVEL_SUCCESS = 1;
	private final static byte END_LEVEL_FAILED = 2;
	
	private Image gfx;

	private boolean left, right, fire;
	
	private byte[] encLevels;
	private int indexCurrentLevel;
	private int indexNextLevel;
	private byte[][] level;
	private int indexLevel;
		
	private boolean collision;
	private byte[][] checkBalls; // Grid used to check balls
	private byte removeCheck; // Nb of balls to check next turn
	private byte removeCount; // Nb of balls in the set
	
	private byte pos; // cursor position
	private boolean refreshBall; // nextBall needs refresh
	private byte currentBall; // Ball launched (-1 -> no ball)
	private byte launchBall; // Ball @ launcher
	private byte nextBall; // Ball @ next
	private byte[] countBalls;
	
	private int currentX, currentY; // Current position
	private int previousX, previousY; // Last position (clear)
	private int moveX, moveY; // Movement * 1024
	private int absoluteX, absoluteY; // Position * 1024
	private int posX, posY; // Grid position of ball
	
	private Random rand;
	
	private int compressor;
	private byte changeCompressor;
	
	private byte life;
	private byte nbBalls;
	private byte endOfLevel;
	private int waitLaunch;
	private byte removing;
	
	public Game(MIDlet midlet) {
		super(midlet);
	}
	
	public Game(MIDlet midlet, byte[] storedLevel) {
		super(midlet);
		
		int targetLevel = storedLevel[0] + (storedLevel[1] << 7);
		
		while (targetLevel > indexLevel) {
			if (!setLevel(true)) {
				stopDisplay();
				((MicroFB)getMIDlet()).launchScores(targetLevel);				
			}
		}
		
		life = storedLevel[2];

		launchBall = nextBall();
		nextBall = nextBall();		
	}	
	
	public void initClass() {		
		if (endOfLevel == 0) {
			gfx = ((MicroFB)getMIDlet()).getImage();
			encLevels = ((MicroFB)getMIDlet()).getLevelData();
			launcher = ((MicroFB)getMIDlet()).getLauncherData();
			ballMove = ((MicroFB)getMIDlet()).getMoveData();
			
			rand = new Random(System.currentTimeMillis());
			
			life = 4;
		}
		
		if (endOfLevel == END_LEVEL_FAILED) {
			life--;
		}
		
		level = new byte[8][13];
		if (!setLevel(endOfLevel != END_LEVEL_FAILED)) {
			stopDisplay();
			((MicroFB)getMIDlet()).launchScores(indexLevel);
			countBalls[0] = 1; // Avoid infinite loop
		}

		checkBalls = new byte[8][13];
		removing = 0;
		
		System.gc(); // Just in case...
				
		pos = (byte)17;
		currentBall = -1;
		compressor = 0;
		changeCompressor = 0;
		waitLaunch = 0;
		launchBall = nextBall();
		nextBall = nextBall();
		
		endOfLevel = 0;
	}
	
	private boolean setLevel(boolean next) {
		countBalls = new byte[9];
		
		for (int i=0 ; i<8 ; i++) {
			for (int j=0 ; j<13 ; j++) {
				level[i][j] = 8;
			}
		}
		
		int reference = indexCurrentLevel;
		if (next) {
			reference = indexNextLevel;
		}
		
		if (reference == encLevels.length) {
			// No more level available
			return false; 
		}
		
		int x = 0;
		int y = 0;
		while (encLevels[reference] != -1) {
			if (encLevels[reference] < 0) {
				level[x][y] = 8;
			}
			else {
				level[x][y] = (byte)((encLevels[reference] >> 4) & 7);
			}
			level[x+1][y] = (byte)(encLevels[reference] & 15);
			
			countBalls[level[x][y]]++;
			countBalls[level[x+1][y]]++;
			
			reference++;
			if (reference == encLevels.length) {
				// Last level
				indexLevel++; // Must be next
				return true;
			}
			
			x += 2;
			if (x == 8) {
				x = 0;
				y++;
			}
		}
		reference++;
		if (next) {
			indexLevel++;
			indexCurrentLevel = indexNextLevel;
			indexNextLevel = reference;
		}

		nbBalls = 0;
		for (int i=0 ; i<8 ; i++) {
			nbBalls += countBalls[i];
		}
		
		return true;
	}
	
	
	private byte nextBall() {
		int color = -1;
		
		do {
			color = rand.nextInt() % 8;
			if (color < 0) {
				color += 8;
			}
		}		
		while (countBalls[color] == 0);
		
		return (byte)color;
	}
	
	private void drawBall(Graphics g, int x, int y, int color) {
		int px = x - 8 * color;
		int py = y - 129;
		for (int i=0 ; i<BALL_DEF.length ; i++) {
			g.setClip(x + BALL_DEF[i][0], y + BALL_DEF[i][1], BALL_DEF[i][2], BALL_DEF[i][3]);
			g.drawImage(gfx, px, py, Graphics.TOP | Graphics.LEFT);
		}
	}

	private void clearBall(Graphics g, int x, int y) {
		for (int i=0 ; i<BALL_DEF.length ; i++) {
			g.setClip(x + BALL_DEF[i][0], y + BALL_DEF[i][1], BALL_DEF[i][2], BALL_DEF[i][3]);
			g.drawImage(gfx, 0, 0, Graphics.TOP | Graphics.LEFT);
		}
	}	

	public void initDraw(Graphics g) {
	
                g.setColor(0x000000);
                g.fillRect(0, 0, getWidth(), getHeight());
                // initial draw
		// Background
		g.setClip(0, 0, 128, 96);
		g.drawImage(gfx, 0, 0, Graphics.TOP | Graphics.LEFT);
		
		// life
		for (int i=0 ; i<life ; i++) {
			g.setClip(29 + 8 * i, 2, 7, 7);
			g.drawImage(gfx, -76 + 8 * i, -119, Graphics.TOP | Graphics.LEFT);
		}
		
		// Bubbles
		for (int j=0 ; j<12 ; j++) {
			for (int i=(j%2) ; i<8 ; i++) {
				if (level[i][j] != 8) {
					drawBall(g, 104 - 7 * (j + compressor), 10 + 8 * i - 4 * (j%2), level[i][j]);
				}
			}
		}
		
		// Level
		String name = Integer.toString(indexLevel);
		for (int i=0 ; i<name.length() ; i++) {
			g.setClip(105 - name.length() * 2 + i * 4, 89, 3, 7);
			g.drawImage(gfx, 105 - name.length() * 2 + i * 4 - Integer.parseInt(name.substring(i, i+1)) * 3, -52, Graphics.TOP | Graphics.LEFT);
		}
		
		// Compressor
		g.setClip(112 - 7 * compressor, 10, 2, 64);
		g.drawImage(gfx, -14 - 7 * compressor, -86, Graphics.TOP | Graphics.LEFT);
		g.setClip(114 - 7 * compressor, 19, 8, 14);
		g.drawImage(gfx, -3 - 7 * compressor, -104, Graphics.TOP | Graphics.LEFT);
		g.setClip(114 - 7 * compressor, 51, 8, 14);
		g.drawImage(gfx, -3 - 7 * compressor, -72, Graphics.TOP | Graphics.LEFT);
		for (int i=0 ; i<=compressor ; i++) {
			g.setClip(122 - 7 * i, 24, 7, 4);
			g.drawImage(gfx, 5 - 7 * i, -113, Graphics.TOP | Graphics.LEFT);
			g.setClip(122 - 7 * i, 56, 7, 4);
			g.drawImage(gfx, 5 - 7 * i, -81, Graphics.TOP | Graphics.LEFT);
		}

		// Next ball
		g.setClip(39, 87, 8, 8);
		g.drawImage(gfx, 39 - 8 * nextBall, -42, Graphics.TOP | Graphics.LEFT);
	}
	
	public void draw(Graphics g) {
            
               
		// draw penguin
		g.setClip(54, 82, 9, 13);
		if (left) {
			g.drawImage(gfx, -10, -46, Graphics.TOP | Graphics.LEFT);
		}
		else if (right) {
			g.drawImage(gfx, -19, -46, Graphics.TOP | Graphics.LEFT);
		}
		else if (fire) {
			g.drawImage(gfx, -28, -46, Graphics.TOP | Graphics.LEFT);
		}
		else {
			g.drawImage(gfx, 0, 0, Graphics.TOP | Graphics.LEFT);
		}
		
		// clean current ball
		if (currentBall != -1 && !getInit()) {
			clearBall(g, previousX, previousY);
		}		
		
		// clean removed balls
		if (removing != 0) {
			for (int i=0 ; i<8 ; i++) {
				for (int j=0 ; j<12 ; j++) {
					if (checkBalls[i][j] != 0) {
						int val = checkBalls[i][j] - 1;
						int x = 104 - (j + compressor) * 7;
						int y = 10 - (j%2) * 4 + i * 8;
						clearBall(g, x, y);
						if (val != 0) {
							g.setClip(x + 4 - val, y + 4 - val, val << 1, val << 1);
							g.setColor(255, 255, 255);
							g.fillRect(0, 0, 128, 96);			
						}
					}
				}
			}
		}
		
		// Launcher
		g.setClip(3, 26, 19, 33);
		g.drawImage(gfx, 0, 0, Graphics.TOP | Graphics.LEFT);
		g.setClip(3+launcher[pos][0], 26+launcher[pos][1], launcher[pos][2], launcher[pos][3]);
		g.drawImage(gfx, 3+launcher[pos][4], launcher[pos][5] - 70, Graphics.TOP | Graphics.LEFT);
		
		// launch ball
		g.setClip(3, 38, 8, 8);
		g.drawImage(gfx, 3 - 8 * launchBall, -91, Graphics.TOP | Graphics.LEFT);
		
		// next ball
		if (refreshBall) {
			g.setClip(39, 87, 8, 8);
			g.drawImage(gfx, 39 - 8 * nextBall, -42, Graphics.TOP | Graphics.LEFT);			
		}

		// current ball
		if (currentBall != -1) {
			if (removeCount < 3) {
				drawBall(g, currentX, currentY, currentBall);
			}
		}
		
		// Warnings
		if (waitLaunch == 0 | waitLaunch > 100 | changeCompressor == 9) {
			if (waitLaunch % 10 == 0) {
				g.setClip(0, 0, 13, 13);
				if (changeCompressor == 9) {
					g.drawImage(gfx, -111, -203, Graphics.TOP | Graphics.LEFT);
				}
				else {
					g.drawImage(gfx, 0, 0, Graphics.TOP | Graphics.LEFT);
				}
			}
			else if (waitLaunch % 10 == 5) {
				g.setClip(0, 0, 13, 13);
				if (waitLaunch > 100) {
					g.drawImage(gfx, -111, -190, Graphics.TOP | Graphics.LEFT);
				}
				else {
					g.drawImage(gfx, 0, 0, Graphics.TOP | Graphics.LEFT);
				}
			}
		}
		
		
		if (endOfLevel == END_LEVEL_SUCCESS) {
			g.setClip(32, 37, 64, 22);
			g.drawImage(gfx, 32, -179, Graphics.TOP | Graphics.LEFT);			
		}
		else if (endOfLevel == END_LEVEL_FAILED) {
			g.setClip(32, 37, 64, 22);
			g.drawImage(gfx, -32, -179, Graphics.TOP | Graphics.LEFT);			
			
			// Draw last line
			int outPosition = 12 - compressor;
			for (int i=(outPosition%2) ; i<8 ; i++) {
				if (level[i][outPosition] != 8) {
					drawBall(g, 104 - 7 * (outPosition + compressor), 10 + 8 * i - 4 * (outPosition%2), level[i][outPosition]);
				}
			}
		}

        }
	
	public void keyPressed(int value) {
		left |= (getGameAction(value) == UP || value == KEY_NUM2); 
		right |= (getGameAction(value) == DOWN || value == KEY_NUM8);
		fire |= (getGameAction(value) == RIGHT || value == KEY_NUM6);	
	}
	
	public void keyReleased(int value) {
		left ^= (getGameAction(value) == UP || value == KEY_NUM2); 
		right ^= (getGameAction(value) == DOWN || value == KEY_NUM8);
		fire ^= (getGameAction(value) == RIGHT || value == KEY_NUM6);				
	}
	
	public void action() {
		// Init values
		if (collision) {
			collision = false;
			currentBall = -1;
			removeCount = 0;
		}
		refreshBall = false;
	
		// player moves
		if (left & right) {
			left = false;
			right = false;
		}
		
		if (left & (pos > 0)) {
			pos--;
		}
		if (right & (pos < 35)) {
			pos++;
		}
			
		// Check balls to remove
		if (removing != 0) {
			for (int j=0 ; j<13 ; j++) {
				for (int i=0 ; i<8 ; i++) {
					if (checkBalls[i][j] != 0) {
						checkBalls[i][j]--;
						if (checkBalls[i][j] == 0) {
							removing--;
						}
					}
				}
			}
		}
		
		
		if (endOfLevel == 0) {	
			// Check wait launch
			if (currentBall == -1) {
				waitLaunch++;
				
				if (fire | waitLaunch == 150) {
					waitLaunch = 0;
					
					currentBall = launchBall;
					launchBall = nextBall;
					nextBall = nextBall();
					refreshBall = true;
					
					moveX = ballMove[pos][0];
					moveY = ballMove[pos][1];
					absoluteX = 3072;
					absoluteY = 38912;
					
					currentX = 3;
					currentY = 38;
				}
			}
				
			// Ball management
			if (currentBall != -1) {		
				previousX = currentX;
				previousY = currentY;
			
				if (!controlBall()) {
					// No collision occured
					controlBall(); // second half of movement
				}
				
				currentX = absoluteX >> 10;
				currentY = absoluteY >> 10;
			}
		}
	}
	
	
	private boolean controlBall() {
		absoluteX += moveX;
		absoluteY += moveY;
		
		if (absoluteY < 10240) {
			absoluteY = 20480 - absoluteY;
			moveY = -moveY;
		}
		else if (absoluteY > 67584) {
			absoluteY = 135168 - absoluteY;
			moveY = -moveY;
		}
		
		// Test collision
		posX = ((113664 - absoluteX) / 7168) - compressor;
		for (int testX = 0 ; testX <= 1 ; testX++) {
			if (posX < 0) {
				collision = true;
			}
			else if (posX < 12) {
				posY = (absoluteY + ((posX % 2) * 4096) - 10240) >> 13;
				for (int testY = 0 ; testY <= 1 ; testY++) {
					if (posY < 8 && level[posY][posX] != 8) {

						int distX = absoluteX - (106496 - (posX + compressor) * 7168);
						int distY = absoluteY - (10240 - (posX % 2) * 4096 + posY * 8192);
						
						collision |= (distX * distX + distY * distY) < 51380224;
					}
					posY++;
				}
			}
			posX--;
		}
		
		if (collision) {
			// Search for posX
			posX++; // Right part of bubble
			if (posX < 0) {
				posX = 0;
				absoluteX = 106496 - compressor * 7168;
			}
			else {
				int nextX = 106496 - (posX + compressor) * 7168;
									
				if ((nextX - absoluteX) >= 3584) {
					posX += 1;
					nextX -= 7168;
				}
				
				absoluteX = nextX;
			}
			
			// Search for posY
			posY = (absoluteY - 6144 + (posX % 2) * 4096) >> 13;
			absoluteY = 10240 - (posX % 2) * 4096 + posY * 8192;

			// Add bubble
			level[posY][posX] = currentBall;
			countBalls[currentBall]++;
			nbBalls++;
			
			// Check if ball is part of a set (remove)
			byte[][] removeBalls = null;
			if (removing != 0) {
				removeBalls = new byte[8][13];
			}
			else {
				removeBalls = checkBalls;
			}
			
			removeBalls[posY][posX] = CHECK_NEXT_TURN;
			removeCheck = 1;
			removeCount = 0;
			
			while (removeCheck != 0) {
				for (int j=0 ; j<13 ; j++) {
					for (int i=0 ; i<8 ; i++) {
						if (removeBalls[i][j] == CHECK_NEXT_TURN) {
							if (level[i][j] == currentBall) {
								removeBalls[i][j] = PART_OF_SET;
								removeCount++;
								
								// Check neighbors
								checkNeighbors(i, j, removeBalls);
							}
							else {
								removeBalls[i][j] = 0; // No action
							}
							
							removeCheck--;
						}
					}
				}
			}
			
			if (removeCount >= 3) {
				// Ball set is removed
				for (int i=0 ; i<8 ; i++) {
					if (level[i][0] != 8 && removeBalls[i][0] != PART_OF_SET) {
						removeBalls[i][0] = CHECK_NEXT_TURN;
						removeCheck++;
					}
				}
				
				while (removeCheck != 0) {
					for (int j=0 ; j<13 ; j++) {
						for (int i=0 ; i<8 ; i++) {
							if (removeBalls[i][j] == CHECK_NEXT_TURN) {
								removeBalls[i][j] = LINKED_TO_TOP;
								removeCheck--;	
								
								// Check neighbors
								checkNeighbors(i, j, removeBalls);
							}
						}
					}
				}
				
				// Physically remove balls
				for (int j=0 ; j<13 ; j++) {
					for (int i=0 ; i<8 ; i++) {
						if (level[i][j] != 8 
								&& (removeBalls[i][j] == 0 || removeBalls[i][j] == PART_OF_SET)) {
							countBalls[level[i][j]]--;
							level[i][j] = 8;
							removeBalls[i][j] = REMOVE_BALL;
							removing++;
							nbBalls--;
						}		
						else {
							removeBalls[i][j] = 0;
						}
					}
				}
				
				// Complete with previous data (if any)
				// Should hardy ever happen
				if (removeBalls != checkBalls) {
					for (int j=0 ; j<13 ; j++) {
						for (int i=0 ; i<8 ; i++) {
							if (removeBalls[i][j] != 0) {
								if (checkBalls[i][j] != 0) {
									removing--;
								}
								checkBalls[i][j] = removeBalls[i][j];
							}
						}
					}
				}
				
				// Check end of level (Won)
				if (nbBalls == 0) {
					// Store level reached
					try {
						RecordStore rs = RecordStore.openRecordStore(Intro.STATE_DB, true);
						
						byte[] toStore = new byte[3];
						toStore[0] = (byte)((indexLevel+1) & 127);
						toStore[1] = (byte)((indexLevel+1) >> 7);
						toStore[2] = life;
						
						if (rs.getNumRecords() == 0) {
							rs.addRecord(toStore, 0, 3);
						}
						else {
							rs.setRecord(1, toStore, 0, 3);
						}

						rs.closeRecordStore();
					}
					catch(Exception e) {}
					
					endOfLevel = END_LEVEL_SUCCESS;
					addCommand(new Command("> Lvl "+(indexLevel+1), Command.ITEM, 1));
				}				
			}
			else {
				// Check end of level (Lost)
				checkFailed();
				if (endOfLevel == 0) {
					// Reset checkBalls
					for (int j=0 ; j<13 ; j++) {
						for (int i=0 ; i<8 ; i++) {
							checkBalls[i][j] = 0;
						}
					}					
				}
			}			
			
			// Check compressor (only if game has not yet ended)
			if (endOfLevel == 0) {
				changeCompressor++;
				if (changeCompressor == 10) {
					absoluteX -= 7168; // Moves current ball to new location
					
					changeCompressor = 0;
					compressor++;
					setInit(true);
					refreshBall = true;
					
					// Check end of level (Lost)
					checkFailed();					
				}
			}
		}	
		
		return collision;
	}
	
	private void checkFailed() {
		int outPosition = 12 - compressor;
		boolean failed = false;
		for (int i=0 ; i<8 ; i++) {
			failed |= (level[i][outPosition] != 8);
		}
		if (failed) {
			endOfLevel = END_LEVEL_FAILED;
			if (life == 1) {
				addCommand(new Command(COMMAND_OVER, Command.ITEM, 1));
			}
			else {
				addCommand(new Command(COMMAND_TRY, Command.ITEM, 1));
			}
		}		
	}
	
	private void checkNeighbors(int x, int y, byte[][] removeBalls) {
		if (x > 0) {
			changeState(x-1, y, removeBalls);
		}
		
		if (x < 7) {
			changeState(x+1, y, removeBalls);
		}
		
		if (y > 0) {
			changeState(x, y-1, removeBalls);
			if (y % 2 == 0) {
				if (x < 7) {
					changeState(x+1, y-1, removeBalls);
				}
			}
			else {
				if (x > 0) {
					changeState(x-1, y-1, removeBalls);
				}				
			}
		}
		
		if (y < 11) {
			changeState(x, y+1, removeBalls);
			if (y % 2 == 0) {
				if (x < 7) {
					changeState(x+1, y+1, removeBalls);
				}
			}
			else {
				if (x > 0) {
					changeState(x-1, y+1, removeBalls);
				}				
			}
		}
	}
	
	private void changeState(int x, int y, byte[][] removeBalls) {
		if (removeBalls[x][y] == 0 && level[x][y] != 8) {
			removeBalls[x][y] = CHECK_NEXT_TURN;
			removeCheck++;
		}
	}
	
	public void manageCommand(Command c) {
		if (c.getLabel().equals(COMMAND_OVER)) {
			stopDisplay();
			((MicroFB)getMIDlet()).launchScores(indexLevel - 1);
		}
		else { // 'Try again' or '> level xxx' 
			removeCommand(c);
			initClass();
			setInit(true);
		}
	}
}
