/*
 * [[ MicroFB : Frozen Bubble for mobile phones ]]
 *
 * Copyright (c) 2000-2005 Guillaume Cottenceau.
 * Java sourcecode - Copyright (c) 2005 Glenn Sanson.
 *                                 2006 Adrian Kurek 
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Alexis Younes <73lab at free.fr>
 *      (everything but the bubbles)
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Guillaume Cottenceau <guillaume.cottenceau at free.fr>
 *      (design and manage the project, whole Perl sourcecode)
 *
 * Java version:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *
 *          [[ http://glenn.sanson.free.fr/microfb/ ]]
 *          [[ http://www.frozen-bubble.org/   ]]
 *
 * Support all devices:
 *    Adrian Kurek <wpqrek@wp.pl>
 *
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStore;

import java.util.*;

public class Intro extends Base {

	public final static String STATE_DB = "stdb";	
	
	private final static String COMMAND_START = "Start";
	private final static String COMMAND_HIGH = "Best of";
	private final static String COMMAND_NEW = "New game";
	private final static String COMMAND_CONTINUE = "Continue";
	
	private final static String TEXT = "5 microfb 2 frozen bubble for "
		+ "mobile phones 5 original design and programming by guillaume cottenceau "
		+ "5 graphics by alexis younes and amaury amblard4ladurantie "
		+ "5 java version by glenn sanson and adrian kurek wpqrek0wp1pl 5 frozen bubble official web site 2 "
		+ "http233www1frozen4bubble1org 5 microfb official web site 2 "
		+ "http233glenn1sanson1free1fr3microfb 5 send your comments to glenn1sanson0free1fr 5";
	
	private Image gfx;
	
	private int textPosition;
	private byte subMove;
	private int firstIndex;
	
	private Random rand;
	private int switchEyes;
	private boolean eyesClose;
	
	private Command commandStart;
	private Command commandHigh;
	
	private byte[] storedLevel;
	
	public Intro(MIDlet midlet) {
		super(midlet);
	}	
	
	public void initClass() {		
		gfx = ((MicroFB)getMIDlet()).getImage();
		
		textPosition = 108;
		subMove = 0;
		firstIndex = -27;
		
		commandStart = new Command(COMMAND_START, Command.OK, 1);
		commandHigh = new Command(COMMAND_HIGH, Command.CANCEL, 1);
		
		addCommand(commandStart);
		addCommand(commandHigh);
		
		rand = new Random(System.currentTimeMillis());
		switchEyes = 60 + (rand.nextInt() % 30);
		
		// Get continue value (if any)
		try {
			RecordStore rs = RecordStore.openRecordStore(STATE_DB, true);
						
			storedLevel = new byte[3];
			if (rs.getNumRecords() != 0) {
				// A continue value is stored
				rs.getRecord(1, storedLevel, 0);
			}
			else {
				rs.setRecord(1, storedLevel, 0, 3);
			}
			
			rs.closeRecordStore();
		}
		catch(Exception e) {}
	}

	public void initDraw(Graphics g) {
                
                g.setColor(0x000000);
                g.fillRect(0,0, getWidth(), getHeight());

                // Grey background
		g.setClip(0, 0, 128, 96);
		g.setColor(148, 140, 115);
		g.fillRect(0, 0, 128, 96);
		
		// Border
		for (int i=0 ; i<5 ; i++) {
			g.setClip(0, i*21, 10, Math.min(21, 96-i*21));
			g.drawImage(gfx, -111, 21 * i - 148, Graphics.TOP | Graphics.LEFT);
			g.setClip(118, i*21, 10, Math.min(21, 96-i*21));
			g.drawImage(gfx, 7, 21 * i - 169, Graphics.TOP | Graphics.LEFT);			
		}
		
		// Logo
		g.setClip(9, 7, 111, 68);
		g.drawImage(gfx, 9, -141, Graphics.TOP | Graphics.LEFT);
	}
	
	public void draw(Graphics g) {

                // Remove scrolling text
		g.setClip(10, 82, 108, 7);
		g.setColor(148, 140, 115);
		g.fillRect(10, 82, 108, 7);
		
		// Redraw scrolling text
		int index = 10 - subMove;
		int currentChar = firstIndex;
		for (int pos=0 ; pos<28 ; pos++) {
			if (currentChar >= 0 && currentChar < TEXT.length()) {
				int charValue = (int)TEXT.charAt(currentChar);
				if (charValue != 32) {
					if (charValue < 97) {
						charValue += 75;
					}
					
					g.setClip(Math.max(10, index), 82, 3 + Math.min(0, index - 10) + Math.min(0, 115 - index), 7);
					g.drawImage(gfx, 
							index - 30 - 3 * (charValue - 97), 
							-59, 
							Graphics.TOP | Graphics.LEFT);
				}
			}
			index += 4;
			currentChar++;
		}	
		
		// Eyes
		if (switchEyes == 0) {
			g.setClip(47, 45, 10, 5);
			if (eyesClose) {
				g.drawImage(gfx, 9, -141, Graphics.TOP | Graphics.LEFT);				
			}
			else {
				g.drawImage(gfx, -69, -73,Graphics.TOP | Graphics.LEFT);				
			}
		}
	}
		
	public void action() {
		// Move text
		textPosition--;
		subMove++;
		if (subMove == 4) {
			subMove = 0;
			firstIndex++;
			if (firstIndex > TEXT.length()) {
				textPosition = 108;
				firstIndex = -27;				
			}
		}
		
		// Eyes
		if (switchEyes == 0) {
			if (eyesClose) {
				switchEyes = 60 + (rand.nextInt() % 30);
			}
			else {
				switchEyes = 8 + (rand.nextInt() % 4);
			}
			eyesClose = !eyesClose;
		}
		else {
			switchEyes--;
		}
	}
	
		
	public void manageCommand(Command c) {
		if (COMMAND_START.equals(c.getLabel())) {
			if (storedLevel[2] == 0) {
				stopDisplay();
				((MicroFB)getMIDlet()).launchGame();
			}
			else {
				// A continue exist
				// Remove current menu
				removeCommand(commandStart);
				removeCommand(commandHigh);
				
				// Add new set of commands
				addCommand(new Command(COMMAND_NEW, Command.OK, 1));
				addCommand(new Command(COMMAND_CONTINUE, Command.CANCEL, 1));
			}
		}
		else if (COMMAND_HIGH.equals(c.getLabel())) {
			stopDisplay();
			((MicroFB)getMIDlet()).launchScores();
		}
		else if (COMMAND_NEW.equals(c.getLabel())) {
			stopDisplay();
			((MicroFB)getMIDlet()).launchGame();
		}
		else if (COMMAND_CONTINUE.equals(c.getLabel())) {
			stopDisplay();
			((MicroFB)getMIDlet()).launchGame(storedLevel);
		}
	}
}
