/*
 * [[ MicroFB : Frozen Bubble for mobile phones ]]
 *
 * Copyright (c) 2000-2005 Guillaume Cottenceau.
 * Java sourcecode - Copyright (c) 2005 Glenn Sanson.
 *                                 2006 Adrian Kurek 
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Alexis Younes <73lab at free.fr>
 *      (everything but the bubbles)
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Guillaume Cottenceau <guillaume.cottenceau at free.fr>
 *      (design and manage the project, whole Perl sourcecode)
 *
 * Java version:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *
 *          [[ http://glenn.sanson.free.fr/microfb/ ]]
 *          [[ http://www.frozen-bubble.org/   ]]
 *
 * Support all devices:
 *    Adrian Kurek <wpqrek@wp.pl>
 *
 */

import java.io.IOException;
import java.io.InputStream;

import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;
import javax.microedition.lcdui.*;

public class MicroFB extends MIDlet {

	private Display disp;
	
	private Image img;
	private byte[] levelData;
	private byte[][] launcherData;
	private int[][] moveData;
	
	public MicroFB() {
		try {
		    // Get game gfx
			img = Image.createImage ("/gfx.png");

			// Get levels data
			// Get data size (available() does not seem to work with Nokia 6100?)
			InputStream in = getClass().getResourceAsStream("/levels.dat");			
			int size = 0;
			while (in.read() != -1) {
				size++;
			}
			in.close();
			// Get data
			in = getClass().getResourceAsStream("/levels.dat");
			levelData = new byte[size];
			in.read(levelData);
			in.close();
			
			// Get launcher data
			launcherData = new byte[35][6];
			in = getClass().getResourceAsStream("/maths.dat");
			for (int i=0 ; i<launcherData.length ; i++) {
				in.read(launcherData[i]);
			}
			// Get movement data
			moveData = new int[35][2];
			byte[] tempX = new byte[2];
			byte[] tempY = new byte[2];
			for (int i=17 ; i<moveData.length ; i++) {
				in.read(tempX);
				in.read(tempY);
				moveData[i][0] = (tempX[0] << 7) + tempX[1];
				moveData[i][1] = (tempY[0] << 7) + tempY[1];
				moveData[34 - i][0] = moveData[i][0];
				moveData[34 - i][1] = -moveData[i][1];
			}
			in.close();
		}
		catch (IOException e) {};
		
		disp = Display.getDisplay(this);
	}
	
	public Image getImage() {
		return img;
	}
	
	public byte[] getLevelData() {
		return levelData;
	}

	public byte[][] getLauncherData() {
		return launcherData;
	}

	public int[][] getMoveData() {
		return moveData;
	}	
	
	public void launchGame() {
		disp.setCurrent(new Game(this));
	}

	public void launchGame(byte[] level) {
		disp.setCurrent(new Game(this, level));
	}	
	
	public void launchIntro() {
		disp.setCurrent(new Intro(this));
	}
	
	public void launchScores() {
		disp.setCurrent(new Scores(this, 0));
	}
	
	public void launchScores(int level) {
		disp.setCurrent(new Scores(this, level));
	}	
	
	protected void startApp() throws MIDletStateChangeException {
		disp.setCurrent(new Intro(this));
	}

	protected void pauseApp() {
	}

	protected void destroyApp(boolean arg0) throws MIDletStateChangeException {
	}

}
