/*
 * [[ MicroFB : Frozen Bubble for mobile phones ]]
 *
 * Copyright (c) 2000-2005 Guillaume Cottenceau.
 * Java sourcecode - Copyright (c) 2005 Glenn Sanson.
 *                                 2006 Adrian Kurek 
 *
 * This code is distributed under the GNU General Public License 
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 * Artwork:
 *    Alexis Younes <73lab at free.fr>
 *      (everything but the bubbles)
 *    Amaury Amblard-Ladurantie <amaury at linuxfr.org>
 *      (the bubbles)
 *
 * Design & Programming:
 *    Guillaume Cottenceau <guillaume.cottenceau at free.fr>
 *      (design and manage the project, whole Perl sourcecode)
 *
 * Java version:
 *    Glenn Sanson <glenn.sanson at free.fr>
 *
 *          [[ http://glenn.sanson.free.fr/microfb/ ]]
 *          [[ http://www.frozen-bubble.org/   ]]
 *
 * Support all devices:
 *    Adrian Kurek <wpqrek@wp.pl>
 *
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.*;

public class Scores extends Base {
	
	private final static String DB = "db";
	
	private final static byte[][] POSITION_BASE = {{82, 14}, {10, 8}, {46, 2}}; 
	private final static byte[][] POSITION_RANK = {{107, 10}, {98, 9}, {91, 7}}; // {abs, width}
	
	private final static byte[][] DEFAULT_NAMES = {{6, 11, 4, 13, 13, -1, -1, -1}, 
		{0, 24, 14, -1, -1, -1, -1, -1}, {6, 20, 8, 19, -1, -1, -1, -1}};
	private final static byte[][] DEFAULT_LEVELS = {{1, 0}, {10, 0}, {20, 0}}; 
	
	private final static String COMMAND_OK = "End";
	private final static String COMMAND_BACK = "Back";
	
	private final static String MESSAGE_NEW = "5 congratulations111 you are now"
		+ "in the hall of fame 5 please enter your name 5";
	private final static String MESSAGE_HIGH = "5 microfb 2 the hall of fame 5";

	private int textPosition;
	private byte subMove;
	private int firstIndex;	
	
	private boolean divider;
	
	private int[] levelReached;
	private byte[][] levelName;
	
	private Image gfx;
	private byte[] encLevels;
	
	private int currentPos;
	private int currentLevel;
	
	private int newLevel;
	private byte newRank;
	private byte indexLetter;
	
	private boolean nameSet;
	private boolean setStored;
	
	private String scrollingText;
	
	private boolean up, down, left, right;
		
	public Scores(MIDlet midlet, int level) {
		super(midlet);
		newLevel = level;
		
		levelReached = new int[3];
		levelName = new byte[3][8];
		
		// Remove stored level
		if (level != 0) {
			try {
				RecordStore rs = RecordStore.openRecordStore(Intro.STATE_DB, true);
				rs.setRecord(1, new byte[3], 0, 3);
				rs.closeRecordStore();						
			}
			catch(Exception e) {}		
		}
		
		// Retrieve highscores
		try {
			RecordStore rs = RecordStore.openRecordStore(DB, true);
			
			if (rs.getNumRecords() == 0) {
				// Insert default values
				for (int rank = 0 ; rank < 3 ; rank++) {
					rs.addRecord(DEFAULT_LEVELS[rank], 0, DEFAULT_LEVELS[rank].length);
					rs.addRecord(DEFAULT_NAMES[rank], 0, DEFAULT_NAMES[rank].length);
				}
			}
			
			// Read data
			int indexRecord = 1;
			for (int rank = 0 ; rank < 3 ; rank++) {
				byte[] tmpLevel = new byte[2];
				rs.getRecord(indexRecord, tmpLevel, 0);
				levelReached[rank] = tmpLevel[0] + (((int)tmpLevel[1]) << 7);
				indexRecord++;
				rs.getRecord(indexRecord, levelName[rank], 0);
				indexRecord++;
			}
			
			rs.closeRecordStore();
		}
		catch(Exception e) {}
				
		newRank = -1;		
		for (int rank=0 ; rank<3 ; rank++) {
			if (newLevel >= levelReached[rank]) {
				if (rank != 0) {
					levelReached[rank - 1] = levelReached[rank];
					levelName[rank - 1] = levelName[rank];
				}
				levelReached[rank] = newLevel;
				newRank = (byte)rank;
			}
		}
		
		if (newRank != -1) {
			levelName[newRank] = new byte[8];
			for (int i=0 ; i<8 ; i++) {
				levelName[newRank][i] = -1;
			}
			
			this.addCommand(new Command(COMMAND_OK, Command.ITEM, 1));
			
			scrollingText = MESSAGE_NEW;		
		}
		else {
			this.addCommand(new Command(COMMAND_BACK, Command.ITEM, 1));
			
			scrollingText = getMessage();
		}
	}
	
	public void initClass() {		
		gfx = ((MicroFB)getMIDlet()).getImage();
		encLevels = ((MicroFB)getMIDlet()).getLevelData();
		
		textPosition = 108;
		subMove = 0;
		firstIndex = -27;		
		
		currentPos = 0;
		currentLevel = 1;
	}

	private byte[][] getLevel(int level) {
		byte[][] output = new byte[8][12];
		for (int i=0 ; i<8 ; i++) {
			for (int j=0 ; j<12 ; j++) {
				output[i][j] = 8;
			}
		}
		
		while(currentLevel < level) {
			if (encLevels[currentPos] == -1) {
				currentLevel++;
			}
			currentPos++;
		}
		
		int reference = currentPos;
				
		int x = 0;
		int y = 0;
		while (encLevels[reference] != -1) {
			if (encLevels[reference] < 0) {
				output[x][y] = 8;
			}
			else {
				output[x][y] = (byte)((encLevels[reference] >> 4) & 7);
			}
			output[x+1][y] = (byte)(encLevels[reference] & 15);
						
			reference++;
			if (reference == encLevels.length) {
				// Last level
				return output;
			}
			
			x += 2;
			if (x == 8) {
				x = 0;
				y++;
			}
		}
		
		return output;
	}
	
	public void initDraw(Graphics g) {
                g.setColor(0x000000);
                g.fillRect(0,0, getWidth(), getHeight());
		// Grey background
		g.setClip(0, 0, 128, 96);
		g.setColor(148, 140, 115);
		g.fillRect(0, 0, 128, 96);
		
		// Border
		for (int i=0 ; i<5 ; i++) {
			g.setClip(0, i*21, 10, Math.min(21, 96-i*21));
			g.drawImage(gfx, -111, 21 * i - 148, Graphics.TOP | Graphics.LEFT);
			g.setClip(118, i*21, 10, Math.min(21, 96-i*21));
			g.drawImage(gfx, 7, 21 * i - 169, Graphics.TOP | Graphics.LEFT);			
		}
		
		// Highscores
		for (int rank=0 ; rank<3 ; rank++) {
			// Rank
			g.setClip(POSITION_BASE[rank][0] + 14, POSITION_BASE[rank][1], POSITION_RANK[rank][1], 13);
			g.drawImage(gfx, 
					POSITION_BASE[rank][0] + 14 - POSITION_RANK[rank][0],
					POSITION_BASE[rank][1] - 128,
					Graphics.TOP | Graphics.LEFT);
			
			// Level view
			g.setClip(POSITION_BASE[rank][0] + 3, POSITION_BASE[rank][1] + 17, 30, 18);
			g.drawImage(gfx,
					POSITION_BASE[rank][0] - 31,
					POSITION_BASE[rank][1] - 32,
					Graphics.TOP | Graphics.LEFT);
			g.setColor(255, 255, 255);
			g.drawRect(POSITION_BASE[rank][0] + 3, POSITION_BASE[rank][1] + 17, 29, 17);
			g.drawLine(POSITION_BASE[rank][0] + 7, POSITION_BASE[rank][1] + 17, 
					POSITION_BASE[rank][0] + 7, POSITION_BASE[rank][1] + 34);
			
			// Balls
			byte[][] level = getLevel(levelReached[rank]);
			for (int j=0 ; j<12 ; j++) {
				for (int i=j%2 ; i<8 ; i++) {
					if (level[i][j] != 8) {
						g.setClip(POSITION_BASE[rank][0] + 30 - j*2, POSITION_BASE[rank][1] + 18 + i*2 - j%2, 2, 2);
						g.drawImage(gfx,
								POSITION_BASE[rank][0] + 27 - j*2 - level[i][j] * 8,
								POSITION_BASE[rank][1] - 113 + i*2 - j%2,
								Graphics.TOP | Graphics.LEFT);
					}
				}
			}
			
			// Level number
			g.setClip(POSITION_BASE[rank][0] + 5, POSITION_BASE[rank][1] + 37, 26, 15);
			g.drawImage(gfx,
					POSITION_BASE[rank][0] - 86,
					POSITION_BASE[rank][1] - 44,
					Graphics.TOP | Graphics.LEFT);
			String name = Integer.toString(levelReached[rank]);
			for (int i=0 ; i<name.length() ; i++) {
				g.setClip(POSITION_BASE[rank][0] + 19 - name.length() * 2 + i * 4, 
						POSITION_BASE[rank][1] + 45, 3, 7);
				g.drawImage(gfx, POSITION_BASE[rank][0] + 19 - name.length() * 2 + i * 4 - Integer.parseInt(name.substring(i, i+1)) * 3, 
						POSITION_BASE[rank][1] - 96, Graphics.TOP | Graphics.LEFT);
			}		
			
			// Name
			int nameSize = 0;
			for (int i=0 ; i<levelName[rank].length ; i++) {
				if (levelName[rank][i] != -1) {
					nameSize = i+1;
				}
			}
			int base = POSITION_BASE[rank][0] + 18 - (nameSize << 1);
			for (int i=0 ; i<nameSize ; i++) {
				if (levelName[rank][i] != -1) {
					g.setClip(base + i * 4, POSITION_BASE[rank][1] + 54, 3, 7);
					g.drawImage(gfx,
							base + i * 4 - 30 - levelName[rank][i] * 3,
							POSITION_BASE[rank][1] - 87,
							Graphics.TOP | Graphics.LEFT);
				}
			}
		}
	}
	
	public void draw(Graphics g) {


                if (newRank != -1) {
			int base = POSITION_BASE[newRank][0] + 2;
			g.setClip(base, POSITION_BASE[newRank][1] + 54, 31, 10);
			g.setColor(148, 140, 115);
			g.fillRect(0, 0, 128, 96);
			for (int i=0 ; i<8 ; i++) {
				if (levelName[newRank][i] != -1) {
					g.setClip(base + i * 4, POSITION_BASE[newRank][1] + 54, 3, 7);
					g.drawImage(gfx,
							base + i * 4 - 30 - levelName[newRank][i] * 3,
							POSITION_BASE[newRank][1] - 87,
							Graphics.TOP | Graphics.LEFT);
				}
			}
			if (!nameSet) {
				g.setClip(base + indexLetter * 4, POSITION_BASE[newRank][1] + 61, 3, 3);
				g.drawImage(gfx,
						base + indexLetter * 4 - 123,
						POSITION_BASE[newRank][1] - 82,
						Graphics.TOP | Graphics.LEFT);
			}
		}
		
		// Clean up scrolling text
		g.setClip(10, 82, 108, 7);
		g.setColor(148, 140, 115);
		g.fillRect(10, 82, 108, 7);
		
		// Redraw scrolling text
		int index = 10 - subMove;
		int currentChar = firstIndex;
		for (int pos=0 ; pos<28 ; pos++) {
			if (currentChar >= 0 && currentChar < scrollingText.length()) {
				int charValue = (int)scrollingText.charAt(currentChar);
				if (charValue != 32) {
					if (charValue < 97) {
						charValue += 75;
					}
					
					g.setClip(Math.max(10, index), 82, 3 + Math.min(0, index - 10) + Math.min(0, 115 - index), 7);
					g.drawImage(gfx, 
							index - 30 - 3 * (charValue - 97), 
							-59, 
							Graphics.TOP | Graphics.LEFT);
				}
			}
			index += 4;
			currentChar++;
		}		
                
	}
		
	public void action() {
		divider = !divider;
		
		if (newRank != -1 && !nameSet && divider) {
			if (up) {
				levelName[newRank][indexLetter]--;
				if (levelName[newRank][indexLetter] < -1) {
					levelName[newRank][indexLetter] = 30;
				}
			}
			else if (down) {
				levelName[newRank][indexLetter]++;
				if (levelName[newRank][indexLetter] > 30) {
					levelName[newRank][indexLetter] = -1;
				}
			}
			
			if (left) {
				indexLetter--;
				if (indexLetter < 0) {
					indexLetter = 7;
				}
			}
			else if (right) {
				indexLetter++;
				if (indexLetter > 7) {
					indexLetter = 0;
				}
			}
		}		
		
		if (nameSet & !setStored) {
			try {
				RecordStore rs = RecordStore.openRecordStore(DB, true);
				// Write data
				int indexRecord = 1;
				for (int rank = 0 ; rank < 3 ; rank++) {
					byte[] tmpLevel = new byte[2];
					tmpLevel[0] = (byte)(levelReached[rank] & 127);
					tmpLevel[1] = (byte)(levelReached[rank] >> 7);
					rs.setRecord(indexRecord, tmpLevel, 0, tmpLevel.length);
					indexRecord++;
					rs.setRecord(indexRecord, levelName[rank], 0, levelName[rank].length);
					indexRecord++;
				}
				rs.closeRecordStore();
			}
			catch(Exception e) {}
			
			setStored = true;
			this.addCommand(new Command(COMMAND_BACK, Command.ITEM, 1));
		}
		
		// Move text
		textPosition--;
		subMove++;
		if (subMove == 4) {
			subMove = 0;
			firstIndex++;
			if (firstIndex > scrollingText.length()) {
				textPosition = 108;
				firstIndex = -27;				
			}
		}		
	}
	
	private String getMessage() {
		String output = MESSAGE_HIGH;
		
		output += " first ever 2 ";
		int nameSize = 0;
		for (int i=0 ; i<levelName[2].length ; i++) {
			if (levelName[2][i] != -1) {
				nameSize = i+1;
			}
		}
		
		byte[] name = new byte[nameSize];
		for (int i=0 ; i<name.length ; i++) {
			name[i] = (byte)(levelName[2][i] + 97);
		}
		
		output += new String(name).substring(0, nameSize);
		output += " 5 try to beat this highscore111 5";
		
		return output;
	}
	
	public void keyPressed(int value) {
		up |= (getGameAction(value) == UP || value == -1 || value == KEY_NUM2); 
		down |= (getGameAction(value) == DOWN || value == -2 || value == KEY_NUM8);
		left |= (getGameAction(value) == LEFT || value == -3 || value == KEY_NUM4);
		right |= (getGameAction(value) == RIGHT || value == -4 || value == KEY_NUM6);	
	}
	
	public void keyReleased(int value) {
		up ^= (getGameAction(value) == UP || value == -1 || value == KEY_NUM2); 
		down ^= (getGameAction(value) == DOWN || value == -2 || value == KEY_NUM8);
		left ^= (getGameAction(value) == LEFT || value == -3 || value == KEY_NUM4);
		right ^= (getGameAction(value) == RIGHT || value == -4 || value == KEY_NUM6);				
	}
			
	public void manageCommand(Command c) {
		if (c.getLabel().equals(COMMAND_OK)) {
			removeCommand(c);
			nameSet = true;
			
			scrollingText = getMessage();
			textPosition = 108;
			subMove = 0;
			firstIndex = -27;				
		}
		else if (c.getLabel().equals(COMMAND_BACK)) {
			stopDisplay();
			((MicroFB)getMIDlet()).launchIntro();
		}
	}
}
